@@ -4,6 +4,7 @@
 # Description:  Read AIFF meta information
 #
 # Revisions:    01/06/2006 - P. Harvey Created
+#               09/22/2008 - PH Added DjVu support
 #
 # References:   1) http://developer.apple.com/documentation/QuickTime/INMAC/SOUND/imsoundmgr.30.htm#pgfId=3190
 #               2) http://astronomy.swin.edu.au/~pbourke/dataformats/aiff/
@@ -17,7 +18,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::ID3;
 
-$VERSION = '1.01';
+$VERSION = '1.04';
 
 # information for time/date-based tags (time zero is Jan 1, 1904)
 my %timeInfo = (
@@ -28,9 +29,12 @@ my %timeInfo = (
 
 # AIFF info
 %Image::ExifTool::AIFF::Main = (
-    PROCESS_PROC => \&Image::ExifTool::AIFF::ProcessSubChunks,
     GROUPS => { 2 => 'Audio' },
-    NOTES => 'Only the tags decoded by ExifTool are listed in this table.',
+    NOTES => q{
+        Tags extracted from Audio Interchange File Format (AIFF) files.  See
+        L<http://www-mmsp.ece.mcgill.ca/Documents/AudioFormats/AIFF/AIFF.html> for
+        the AIFF specification.
+    },
 #    FORM => 'Format',
     FVER => {
         Name => 'FormatVersion',
@@ -44,10 +48,24 @@ my %timeInfo = (
         Name => 'Comment',
         SubDirectory => { TagTable => 'Image::ExifTool::AIFF::Comment' },
     },
-    NAME => 'Name',
-    AUTH => { Name => 'Author', Groups => { 2 => 'Author' } },
-   '(c) ' => { Name => 'Copyright', Groups => { 2 => 'Author' } },
-    ANNO => 'Annotation',
+    NAME => {
+        Name => 'Name',
+        ValueConv => '$self->Decode($val, "MacRoman")',
+    },
+    AUTH => {
+        Name => 'Author',
+        Groups => { 2 => 'Author' },
+        ValueConv => '$self->Decode($val, "MacRoman")',
+    },
+   '(c) ' => {
+        Name => 'Copyright',
+        Groups => { 2 => 'Author' },
+        ValueConv => '$self->Decode($val, "MacRoman")',
+    },
+    ANNO => {
+        Name => 'Annotation',
+        ValueConv => '$self->Decode($val, "MacRoman")',
+    },
    'ID3 ' => {
         Name => 'ID3',
         SubDirectory => {
@@ -83,6 +101,11 @@ my %timeInfo = (
             sowt => 'Little-endian, no compression',
         },
     },
+    11 => { #PH
+        Name => 'CompressorName',
+        Format => 'pstring',
+        ValueConv => '$self->Decode($val, "MacRoman")',
+    },
 );
 
 %Image::ExifTool::AIFF::FormatVers = (
@@ -96,7 +119,10 @@ my %timeInfo = (
     GROUPS => { 2 => 'Audio' },
     0 => { Name => 'CommentTime', %timeInfo },
     1 => 'MarkerID',
-    2 => 'Comment',
+    2 => {
+        Name => 'Comment',
+        ValueConv => '$self->Decode($val, "MacRoman")',
+    },
 );
 
 #------------------------------------------------------------------------------
@@ -136,18 +162,34 @@ sub ProcessAIFF($$)
 {
     my ($exifTool, $dirInfo) = @_;
     my $raf = $$dirInfo{RAF};
-    my $verbose = $exifTool->Options('Verbose');
-    my ($buff, $err);
+    my ($buff, $err, $tagTablePtr, $page, $type);
 
     # verify this is a valid AIFF file
     return 0 unless $raf->Read($buff, 12) == 12;
-    return 0 unless $buff =~ /^FORM....(AIF(F|C))/s;
-    $exifTool->SetFileType($1);
-    SetByteOrder('MM');
-    my $tagTablePtr = GetTagTable('Image::ExifTool::AIFF::Main');
     my $pos = 12;
+    # check for DjVu image
+    if ($buff =~ /^AT&TFORM/) {
+        # http://www.djvu.org/
+        # http://djvu.sourceforge.net/specs/djvu3changes.txt
+        my $buf2;
+        return 0 unless $raf->Read($buf2, 4) == 4 and $buf2 =~ /^(DJVU|DJVM)/;
+        $pos += 4;
+        $buff = substr($buff, 4) . $buf2;
+        $tagTablePtr = GetTagTable('Image::ExifTool::DjVu::Main');
+        $exifTool->SetFileType('DJVU');
+        # modifiy FileType to indicate a multi-page document
+        $exifTool->{VALUE}->{FileType} .= " (multi-page)" if $buf2 eq 'DJVM';
+        $type = 'DjVu';
+    } else {
+        return 0 unless $buff =~ /^FORM....(AIF(F|C))/s;
+        $exifTool->SetFileType($1);
+        $tagTablePtr = GetTagTable('Image::ExifTool::AIFF::Main');
+        $type = 'AIFF';
+    }
+    SetByteOrder('MM');
+    my $verbose = $exifTool->Options('Verbose');
 #
-# Read chunks in AIFF image
+# Read through the IFF chunks
 #
     for (;;) {
         $raf->Read($buff, 8) == 8 or last;
@@ -158,19 +200,30 @@ sub ProcessAIFF($$)
         # AIFF chunks are padded to an even number of bytes
         my $len2 = $len + ($len & 0x01);
         if ($tagInfo) {
-            $raf->Read($buff, $len2) == $len2 or $err=1, last;
+            if ($$tagInfo{TypeOnly}) {
+                $len = $len2 = 4;
+                $page = ($page || 0) + 1;
+                $exifTool->VPrint(0, $exifTool->{INDENT} . "Page $page:\n");
+            }
+            $raf->Read($buff, $len2) >= $len or $err=1, last;
+            unless ($$tagInfo{SubDirectory} or $$tagInfo{Binary}) {
+                $buff =~ s/\0+$//;  # remove trailing nulls
+            }
             $exifTool->HandleTag($tagTablePtr, $tag, $buff,
                 DataPt => \$buff,
                 DataPos => $pos,
                 Start => 0,
                 Size => $len,
             );
+        } elsif ($verbose > 2 and $len2 < 1024000) {
+            $raf->Read($buff, $len2) == $len2 or $err = 1, last;
+            Image::ExifTool::HexDump(\$buff, undef, MaxLen => 512);
         } else {
             $raf->Seek($len2, 1) or $err=1, last;
         }
         $pos += $len2;
     }
-    $err and $exifTool->Warn('Error reading AIFF file (corrupted?)');
+    $err and $exifTool->Warn("Error reading $type file (corrupted?)");
     return 1;
 }
 
@@ -193,7 +246,7 @@ information from AIFF (Audio Interchange File Format) audio files.
 
 =head1 AUTHOR
 
-Copyright 2003-2007, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
@@ -216,4 +269,3 @@ L<Image::ExifTool::TagNames/AIFF Tags>,
 L<Image::ExifTool(3pm)|Image::ExifTool>
 
 =cut
-
