@@ -13,13 +13,13 @@ use vars qw($VERSION);
 use Image::ExifTool;    # only for FinishTiffDump()
 use Image::ExifTool::HTML qw(EscapeHTML);
 
-$VERSION = '1.25';
+$VERSION = '1.31';
 
 sub DumpTable($$$;$$$$$);
 sub Open($$$;@);
 sub Write($@);
 
-my ($bkgStart, $bkgEnd, $bkgSpan);
+my ($bkgStart, $bkgEnd, @bkgSpan);
 
 my $htmlHeader1 = <<_END_PART_1_;
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN"
@@ -60,10 +60,12 @@ my $htmlHeader2 = <<_END_PART_2_;
     visibility: hidden;
     position: absolute;
     background: #ffffdd;
-    opacity: 0.8;
+    zoom: 1;
     -moz-opacity: 0.8;
-    filter: alpha(opacity=80);
+    -khtml-opacity: 0.8;
     -ms-filter: 'progid:DXImageTransform.Microsoft.Alpha(Opacity=80)';
+    filter: alpha(opacity=80);
+    opacity: 0.8;
     z-index: 2;
 }
 /* table styles */
@@ -234,15 +236,17 @@ sub new
 # Add information to dump
 # Inputs: 0) HTML dump hash ref, 1) absolute offset in file, 2) data size,
 #         3) comment string, 4) tool tip (or SAME to use previous tip),
-#         5) bit flags (see below), 6) true to use same tooltip as last call
+#         5) bit flags (see below)
 # Bits: 0x01 - print at start of line
 #       0x02 - print red address
 #       0x04 - maker notes data ('M'-class span)
 #       0x08 - limit block length
+#       0x10 - allow double references
+#       0x100 - (reserved)
 # Notes: Block will be shown in 'unused' color if comment string begins with '['
-sub Add($$$$;$$)
+sub Add($$$$;$)
 {
-    my ($self, $start, $size, $msg, $tip, $flag, $sameTip) = @_;
+    my ($self, $start, $size, $msg, $tip, $flag) = @_;
     my $block = $$self{Block};
     $$block{$start} or $$block{$start} = [ ];
     my $htip;
@@ -268,7 +272,9 @@ sub Add($$$$;$$)
 #         2) data pointer, 3) data position, 4) output file or scalar reference,
 #         5) limit level (1-3), 6) title
 # Returns: non-zero if useful output was generated,
-#          or -1 on error loading data and ERROR is set to offending data name
+#          or -1 on error loading data and "ERROR" is set to offending data name
+# Note: The "Error" member may be set externally to print a specific error
+#       message instead of doing the dump.
 sub Print($$;$$$$$)
 {
     local $_;
@@ -280,7 +286,6 @@ sub Print($$;$$$$$)
     $title = 'HtmlDump' unless $title;
     $level or $level = 0;
     my $tell = $raf->Tell();
-    my @starts = sort { $a <=> $b } keys %$block;
     my $pos = 0;
     my $dataEnd = $dataPos + ($dataPt ? length($$dataPt) : 0);
     # initialize member variables
@@ -304,22 +309,36 @@ sub Print($$;$$$$$)
         $self->{Closed}->[$i] = { ID => [ ], Element => { } };
     }
     $bkgStart = $bkgEnd = 0;
-    $bkgSpan = '';
+    undef @bkgSpan;
     my $index = 0;  # initialize tooltip index
-    my (@names, $wasUnused);
-    for ($i=0; $i<@starts; ++$i) {
+    my (@names, $wasUnused, @starts);
+    # only do dump if we didn't have a serious error
+    @starts = sort { $a <=> $b } keys %$block unless $$self{Error};
+    for ($i=0; $i<=@starts; ++$i) {
         my $start = $starts[$i];
-        my $parmList = $$block{$start};
+        my $parmList;
+        if (defined $start) {
+            $parmList = $$block{$start};
+        } elsif ($bkgEnd and $pos < $bkgEnd and not defined $wasUnused) {
+            $start = $bkgEnd;   # finish last bkg block
+        } else {
+            last;
+        }
         my $len = $start - $pos;
         if ($len > 0 and not $wasUnused) {
-            # we have an unused bytes before this data block
-            --$i;           # dump the data block next time around
-            # split unused data into 2 blocks if it spans end of makernotes
-            if (not defined $wasUnused and $bkgEnd and
-                $pos < $bkgEnd + $dataPos and $pos + $len > $bkgEnd + $dataPos)
-            {
+            # we have a unused bytes before this data block
+            --$i;   # dump the data block next time around
+            # split unused data into 2 blocks if it spans end of a bkg block
+            my ($nextBkgEnd, $bkg);
+            if (not defined $wasUnused and $bkgEnd) {
+                foreach $bkg (@bkgSpan) {
+                    next if $pos >= $$bkg{End} + $dataPos or $pos + $len <= $$bkg{End} + $dataPos;
+                    $nextBkgEnd = $$bkg{End} unless $nextBkgEnd and $nextBkgEnd < $$bkg{End};
+                }
+            }
+            if ($nextBkgEnd) {
                 $start = $pos;
-                $len = $bkgEnd + $dataPos - $pos;
+                $len = $nextBkgEnd + $dataPos - $pos;
                 $wasUnused = 0;
             } else {
                 $start = $pos;  # dump the unused bytes now
@@ -347,11 +366,15 @@ sub Print($$;$$$$$)
                 $names[$tipNum] = $name if defined $tipNum;
                 ++$index;
             }
-            if ($flag == 4) {
-                $bkgStart = $start - $dataPos;
-                # (allow for nested makernotes data)
-                $bkgEnd = $bkgStart + $len unless $bkgEnd and $bkgEnd > $bkgStart + $len;
-                $bkgSpan = "<span class='$name M'>";
+            if ($flag & 0x14) {
+                my %bkg = (
+                    Class => $flag & 0x04 ? "$name M" : $name,
+                    Start => $start - $dataPos,
+                    End   => $start - $dataPos + $len,
+                );
+                push @bkgSpan, \%bkg;
+                $bkgStart = $bkg{Start} unless $bkgStart and $bkgStart < $bkg{Start};
+                $bkgEnd = $bkg{End} unless $bkgEnd and $bkgEnd > $bkg{End};
                 push @{$self->{MSpanList}}, $name;
                 next;
             }
@@ -379,7 +402,7 @@ sub Print($$;$$$$$)
                                 # reset $len to the actual length of available data
                                 $raf->Seek(0, 2);
                                 $len = $raf->Tell() - $start;
-                                $tip .= "\nError: Only $len bytes available!" if $tip;
+                                $tip .= "<br>Error: Only $len bytes available!" if $tip;
                                 next;
                             }
                             $buff .= $buf2;
@@ -387,6 +410,8 @@ sub Print($$;$$$$$)
                         }
                     } else {
                         $err = $msg;
+                        $len = length $buff;
+                        $tip .= "<br>Error: Only $len bytes available!" if $tip;
                     }
                 }
                 last;
@@ -433,8 +458,8 @@ sub Print($$;$$$$$)
         delete $$self{TipList};
         $rtnVal = 1;
     } else {
-        Write($outfile, "$title</title></head><body>\n",
-                        "No EXIF or TIFF information found in image\n");
+        my $err = $$self{Error} || 'No EXIF or TIFF information found in image';
+        Write($outfile, "$title</title></head><body>\n$err\n");
         $rtnVal = 0;
     }
     Write($outfile, "</body></html>\n");
@@ -590,7 +615,21 @@ sub DumpTable($$$;$$$$$)
     }
     # loop through each column of hex numbers
     for (;;) {
-        $self->Open('bkg', ($p>=$bkgStart and $p<$bkgEnd) ? $bkgSpan : '', 1, 2);
+        my (@spanClass, @spanCont, $spanClose, $bkg);
+        if ($p >= $bkgStart and $p < $bkgEnd) {
+            foreach $bkg (@bkgSpan) {
+                next unless $p >= $$bkg{Start} and $p < $$bkg{End};
+                push @spanClass, $$bkg{Class};
+                if ($p + 1 == $$bkg{End}) {
+                    $spanClose = 1;
+                } else {
+                    push @spanCont, $$bkg{Class};   # this span continues
+                }
+            }
+            $self->Open('bkg', @spanClass ? "<span class='@spanClass'>" : '', 1, 2);
+        } else {
+            $self->Open('bkg', '', 1, 2);
+        }
         $self->Open('a', $name, 1, 2);
         my $ch = substr($$blockPt,$p-$pos-$skipped,1);
         $c[1] .= sprintf("%.2x", ord($ch));
@@ -603,9 +642,10 @@ sub DumpTable($$$;$$$$$)
         ++$p;
         ++$cols;
         # close necessary elements
-        if ($p >= $bkgEnd) {
+        if ($spanClose) {
+            my $spanCont = @spanCont ? "<span class='@spanCont'>" : '';
             # close without reopening if closing anchor later
-            my $arg = ($p - $pos >= $len) ? 0 : '';
+            my $arg = ($p - $pos >= $len) ? 0 : $spanCont;
             $self->Open('bkg', $arg, 1, 2);
         }
         if ($dblRef and $p >= $endPos) {
@@ -640,7 +680,7 @@ sub DumpTable($$$;$$$$$)
                 my $n = ($len - $p + $pos - $lim) & ~0x0f;
                 if ($n > 16) { # (no use just cutting out one line)
                     $self->Open('bkg', '', 1, 2); # no underline
-                    my $note = "[snip $n bytes]";
+                    my $note = sprintf "[snip %d lines]", $n / 16;
                     $note = (' ' x (24-length($note)/2)) . $note;
                     $c[0] .= "  ...\n";
                     $c[1] .= $note . (' ' x (48-length($note))) . "\n";
@@ -695,6 +735,8 @@ sub FinishTiffDump($$$)
         OtherImageStart   => 'OtherImageLength',
         ImageOffset       => 'ImageByteCount',
         AlphaOffset       => 'AlphaByteCount',
+        MPImageStart      => 'MPImageLength',
+        IDCPreviewStart   => 'IDCPreviewLength',
     );
 
     # add TIFF data to html dump
@@ -727,10 +769,13 @@ sub FinishTiffDump($$$)
             my $name = Image::ExifTool::GetTagName($key);
             my $grp1 = $exifTool->GetGroup($key, 1);
             my $info2 = $exifTool->GetInfo($offsetPair{$tag}, { Group1 => $grp1 });
-            next unless %$info2;
-            my ($key2) = keys %$info2;
+            my $key2 = $offsetPair{$tag};
+            $key2 .= $1 if $key =~ /( .*)/; # use same instance number as $tag
+            next unless $$info2{$key2};
             my $offsets = $$info{$key};
             my $byteCounts = $$info2{$key2};
+            # ignore primary MPImage (this is the whole JPEG)
+            next if $tag eq 'MPImageStart' and $offsets eq '0';
             # (long lists may be SCALAR references)
             my @offsets = split ' ', (ref $offsets ? $$offsets : $offsets);
             my @byteCounts = split ' ', (ref $byteCounts ? $$byteCounts : $byteCounts);
@@ -824,7 +869,7 @@ This module contains code used to generate an HTML-based hex dump of
 information for debugging purposes.  This is code is called when the
 ExifTool 'HtmlDump' option is used.
 
-Currently, only EXIF and TIFF information is dumped.
+Currently, only EXIF/TIFF and JPEG information is dumped.
 
 =head1 BUGS
 
@@ -833,12 +878,11 @@ may run extremely slowly when processing large files with this version of
 Perl.
 
 An HTML 4 compliant browser is needed to properly display the generated HTML
-page, but note that some of these browsers (like Mozilla) may not properly
-display linefeeds in the tool tips.
+page.
 
 =head1 AUTHOR
 
-Copyright 2003-2009, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
