@@ -23,7 +23,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.30';
+$VERSION = '1.32';
 
 sub ProcessKodakIFD($$$);
 sub ProcessKodakText($$$);
@@ -754,12 +754,14 @@ sub WriteKodakIFD($$$);
         Condition => '$$self{Make} =~ /Kodak/i and $$valPt =~ /^([A-Z0-9]{1,11}\0|[A-Z0-9]{12})/i',
         Format => 'string[12]',
         Notes => 'Kodak only',
+        Writable => 0,
     },
     0xc4 => {
         Name => 'UnknownNumber', # (confirmed NOT to be serial number for Easyshare Mini - PH)
         Condition => '$$self{Make} =~ /Kodak/i and $$valPt =~ /^([A-Z0-9]{1,11}\0|[A-Z0-9]{12})/i',
         Format => 'string[12]',
         Notes => 'Kodak only',
+        Writable => 0,
     },
 );
 
@@ -955,11 +957,11 @@ sub WriteKodakIFD($$$);
         Writable => 'int16u',
     },
     0xfa57 => {
-        Name => 'PreviewWidth',
+        Name => 'PreviewImageWidth',
         Writable => 'int16u',
     },
     0xfa58 => {
-        Name => 'PreviewHeight',
+        Name => 'PreviewImageHeight',
         Writable => 'int16u',
     },
 );
@@ -1985,7 +1987,7 @@ interpret Kodak maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
