@@ -15,7 +15,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.03';
+$VERSION = '1.05';
 
 sub ProcessItemID($$$);
 sub ProcessLinkInfo($$$);
@@ -24,6 +24,7 @@ sub ProcessLinkInfo($$$);
 %Image::ExifTool::LNK::Main = (
     PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
     GROUPS => { 2 => 'Other' },
+    VARS => { HEX_ID => 1 },    # print hex ID's in documentation
     NOTES => 'Information extracted from MS Shell Link (Windows shortcut) files.',
     # maybe the Flags aren't very useful to the user (since they are
     # mainly structural), but extract them anyway for completeness
@@ -379,7 +380,7 @@ sub ProcessLinkInfo($$$);
         Format => 'undef[64]',
         RawConv => q{
             $val = $self->Decode($val, 'UCS2');
-            $val =~ s/\0.*//;
+            $val =~ s/\0.*//s;
             return length($val) ? $val : undef;
         },
     },
@@ -473,7 +474,7 @@ sub ProcessItemID($$$)
         my $size = Get16u($dataPt, $pos);
         last if $size < 2 or $pos + $size > $dataLen;
         my $tag = Get16u($dataPt, $pos+2); # (just a guess -- may not be a tag at all)
-        Image::ExifTool::AddTagToTable($tagTablePtr, $tag, {
+        AddTagToTable($tagTablePtr, $tag, {
             Name => sprintf('Item%.4x', $tag),
             SubDirectory => { TagTable => 'Image::ExifTool::LNK::UnknownData' },
         }) unless $$tagTablePtr{$tag};
@@ -698,7 +699,7 @@ information MS Shell Link (Windows shortcut) files.
 
 =head1 AUTHOR
 
-Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
