@@ -19,6 +19,7 @@
 #              14) Carl Bretteville private communication (M9)
 #              15) Zdenek Mihula private communication (TZ8)
 #              16) Olaf Ulrich private communication
+#              17) http://u88.n24.queensu.ca/exiftool/forum/index.php/topic,4922.0.html
 #              JD) Jens Duttke private communication (TZ3,FZ30,FZ50)
 #------------------------------------------------------------------------------
 
@@ -29,7 +30,7 @@ use vars qw($VERSION %leicaLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.67';
+$VERSION = '1.72';
 
 sub ProcessPanasonicType2($$$);
 sub WhiteBalanceConv($;$$);
@@ -99,6 +100,7 @@ sub WhiteBalanceConv($;$$);
     39 => 'Macro-Elmar-M 90mm f/4',         # 11633/11634
     '39 0' => 'Tele-Elmar-M 135mm f/4 (II)',# 11861
     40 => 'Macro-Adapter M',                # 14409
+    '41 3' => 'Apo-Summicron-M 50mm f/2 Asph', #16
     42 => 'Tri-Elmar-M 28-35-50mm f/4 ASPH.',# 11625
     '42 1' => 'Tri-Elmar-M 28-35-50mm f/4 ASPH. (at 28mm)',
     '42 2' => 'Tri-Elmar-M 28-35-50mm f/4 ASPH. (at 35mm)',
@@ -115,7 +117,6 @@ sub WhiteBalanceConv($;$$);
     '51 2' => 'Super-Elmar-M 14mm f/3.8 Asph', # ? (ref 16)
     52 => 'Super-Elmar-M 18mm f/3.8 ASPH.', # ? (ref PH/11)
     '53 2' => 'Apo-Telyt-M 135mm f/3.4', #16
-    '53 3' => 'Apo-Summicron-M 50mm f/2 Asph', #16
 );
 
 # M9 frame selector bits for each lens
@@ -214,6 +215,9 @@ my %shootingMode = (
     55 => 'Handheld Night Shot', #PH (FZ47)
     57 => '3D', #PH (3D1)
     59 => 'Creative Control', #PH (FZ47)
+    62 => 'Panorama', #17
+    63 => 'Glass Through', #17
+    64 => 'HDR', #17
     66 => 'Digital Filter', #PH (GF5 "Impressive Art", "Cross Process", "Color Select", "Star")
 );
 
@@ -226,8 +230,10 @@ my %shootingMode = (
         Name => 'ImageQuality',
         Writable => 'int16u',
         PrintConv => {
+            1 => 'TIFF', #PH (FZ20)
             2 => 'High',
             3 => 'Normal',
+            # 5 - seen this for 1920x1080, 30fps SZ7 video - PH
             6 => 'Very High', #3 (Leica)
             7 => 'Raw', #3 (Leica)
             9 => 'Motion Picture', #PH (LZ6)
@@ -302,7 +308,9 @@ my %shootingMode = (
                 '16'    => 'Normal?', # (only mode for DMC-LC20)
                 '16 0'  => '1-area', # (FZ8)
                 '16 16' => '1-area (high speed)', # (FZ8)
-                '32 0'  => 'Auto or Face Detect', # (Face Detect for FS7, Auto is DMC-L1 guess)
+                # '32 0' is Face Detect for FS7, and Face Detect or Focus Tracking
+                # for the DMC-FZ200 (ref 17), and Auto is DMC-L1 guess,
+                '32 0'  => 'Tracking',
                 '32 1'  => '3-area (left)?', # (DMC-L1 guess)
                 '32 2'  => '3-area (center)?', # (DMC-L1 guess)
                 '32 3'  => '3-area (right)?', # (DMC-L1 guess)
@@ -317,7 +325,7 @@ my %shootingMode = (
             2 => 'On, Mode 1',
             3 => 'Off',
             4 => 'On, Mode 2',
-            # GF1 also has a mode 3
+            # GF1 also has a "Mode 3"
         },
     },
     0x1c => {
@@ -364,6 +372,10 @@ my %shootingMode = (
         Name => 'FlashBias',
         Format => 'int16s',
         Writable => 'int16s',
+        ValueConv => '$val / 3', #17 (older models may not have factor of 3? - PH)
+        ValueConvInv => '$val * 3',
+        PrintConv => 'Image::ExifTool::Exif::PrintFraction($val)',
+        PrintConvInv => 'Image::ExifTool::Exif::ConvertFraction($val)',
     },
     0x25 => { #PH
         Name => 'InternalSerialNumber',
@@ -441,7 +453,7 @@ my %shootingMode = (
         PrintConv => {
             0 => 'Off',
             1 => 'On', #PH (TZ5) [was "Low/High Quality" from ref 4]
-            2 => 'Infinite',
+            2 => 'Auto Exposure Bracketing (AEB)', #17
             4 => 'Unlimited', #PH (TZ5)
         },
     },
@@ -477,7 +489,7 @@ my %shootingMode = (
         },{
             Name => 'ContrastMode',
             Condition => '$$self{Model} =~ /^DMC-(GF\d+|G2)$/',
-            Notes => 'these values are used by the G2, GF1, GF2, GF3 and GF5',
+            Notes => 'these values are used by the G2, GF1, GF2, GF3, GF5 and GF6',
             Writable => 'int16u',
             PrintConv => { # (decoded for GF1 unless otherwise noted)
                 0 => '-2',
@@ -486,7 +498,7 @@ my %shootingMode = (
                 3 => '+1',
                 4 => '+2',
                 # Note: Other Contrast tags will be "Normal" in any of these modes:
-                # 5 - seen for Portrait (FX80)
+                5 => 'Normal 2', # 5 - seen for Portrait (FX80) and Normal (GF6)
                 7 => 'Nature (Color Film)', # (GF1,G2; GF3 "Miniature")
                 9 => 'Expressive', #(GF3)
                 12 => 'Smooth (Color Film) or Pure (My Color)', #(GF1,G2 "Smooth Color")
@@ -506,6 +518,16 @@ my %shootingMode = (
                 47 => 'Dynamic Mono', #(GF5)
                 50 => 'Impressive Art', #(GF5)
                 51 => 'Cross Process', #(GF5)
+                107 => 'Expressive 2', #(GF6)
+                122 => 'Dynamic Monochrome', #(GF6)
+                # more new modes for GF6:
+                # ? => 'Old Days',
+                # ? => 'Toy Pop',
+                # ? => 'Bleach Bypass',
+                # ? => 'Fantasy',
+                # ? => 'Star Filter',
+                # ? => 'One Point Color',
+                # ? => 'Sunshine',
             },
         },{
             Name => 'ContrastMode',
@@ -542,6 +564,7 @@ my %shootingMode = (
             1 => 'Off',
             2 => '10 s',
             3 => '2 s',
+            4 => '10 s / 3 pictures', #17
         },
     },
     # 0x2f - values: 1 (LZ6,FX10K)
@@ -703,6 +726,15 @@ my %shootingMode = (
         Writable => 'int16u',
         Notes => 'positive is a shift toward green',
     },
+    0x48 => { #17
+        Name => 'FlashCurtain',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'n/a',
+            1 => '1st',
+            2 => '2nd',
+        },
+    },
     # 0x48 - int16u: 0
     # 0x49 - int16u: 2
     # 0x4a - int16u: 0
@@ -737,14 +769,17 @@ my %shootingMode = (
     0x51 => {
         Name => 'LensType',
         Writable => 'string',
+        ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
     },
     0x52 => { #7 (DMC-L1)
         Name => 'LensSerialNumber',
         Writable => 'string',
+        ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
     },
     0x53 => { #7 (DMC-L1)
         Name => 'AccessoryType',
         Writable => 'string',
+        ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
     },
     # 0x54 - string[14]: "0000000"
     # 0x55 - int16u: 1
@@ -878,6 +913,19 @@ my %shootingMode = (
         PrintConv => 'sprintf("%.1f kPa",$val)',
         PrintConvInv => '$val=~s/ ?kPa//i; $val',
     },
+    0x0089 => {
+        Name => 'PhotoStyle',
+        Writable => 'int16u',
+        PrintConv => {
+            0 => 'Auto',
+            1 => 'Standard or Custom',
+            2 => 'Vivid',
+            3 => 'Natural',
+            4 => 'Monochrome',
+            5 => 'Scenery',
+            6 => 'Portrait',
+        },
+    },
     0x0e00 => {
         Name => 'PrintIM',
         Description => 'Print Image Matching',
@@ -1376,6 +1424,7 @@ my %shootingMode = (
     0x303 => {
         Name => 'LensType',
         Writable => 'string',
+        ValueConv => '$val=~s/ +$//; $val', # trim trailing spaces
     },
     # 0x340 - same as 0x302
 );
@@ -1549,7 +1598,7 @@ my %shootingMode = (
     },
     0x4068 => {
         Name => 'ExifData',
-        Condition => '$$valPt =~ /^\xff\xd8\xff\xe1..Exif\0\0/',
+        Condition => '$$valPt =~ /^\xff\xd8\xff\xe1..Exif\0\0/s',
         SubDirectory => {
             TagTable => 'Image::ExifTool::Exif::Main',
             ProcessProc => \&Image::ExifTool::ProcessTIFF,
@@ -1684,7 +1733,7 @@ sub ProcessLeicaTrailer($;$)
     delete $$exifTool{LeicaTrailer} if $trailPos;   # done after this
     unless ($len > 0) {
         $exifTool->Warn('Missing Leica MakerNote trailer', 1) if $trailPos;
-        undef $$exifTool{LeicaTrailer};
+        delete $$exifTool{LeicaTrailer};
         return undef;
     }
     my $oldPos = $raf->Tell();
@@ -1836,7 +1885,7 @@ Panasonic and Leica maker notes in EXIF information.
 
 =head1 AUTHOR
 
-Copyright 2003-2012, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
