@@ -27,7 +27,7 @@ use strict;
 use vars qw($VERSION $AUTOLOAD $iptcDigestInfo);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.40';
+$VERSION = '1.41';
 
 sub ProcessPhotoshop($$$);
 sub WritePhotoshop($$$);
@@ -67,7 +67,7 @@ my %psdMap = (
     },
     0x03ee => {
         Name => 'AlphaChannelsNames',
-        PrintConv => 'Image::ExifTool::Photoshop::ConvertPascalString($val)',
+        ValueConv => 'Image::ExifTool::Photoshop::ConvertPascalString($self,$val)',
     },
     0x03ef => { Unknown => 1, Name => 'DisplayInfo' },
     0x03f0 => { Unknown => 1, Name => 'PStringCaption' },
@@ -183,9 +183,12 @@ my %psdMap = (
         Writable => 'string',
         Protected => 1,
         Notes => q{
-            when writing, special values of "new" and "old" represent the digests of the
-            IPTC from the output and original files respectively, and are undefined if
-            the IPTC does not exist in the respective file
+            this tag indicates provides a way for XMP-aware applications to indicate
+            that the XMP is synchronized with the IPTC.  When writing, special values of
+            "new" and "old" represent the digests of the IPTC from the edited and
+            original files respectively, and are undefined if the IPTC does not exist in
+            the respective file.  Set this to "new" as an indication that the XMP is
+            synchronized with the IPTC
         },
         # also note the 'new' feature requires that the IPTC comes before this tag is written
         ValueConv => 'unpack("H*", $val)',
@@ -211,13 +214,12 @@ my %psdMap = (
     # 0x07d0-0x0bb6 Path information
     0x0bb7 => {
         Name => 'ClippingPathName',
-        # convert from a Pascal string
+        # convert from a Pascal string (ignoring 6 bytes of unknown data after string)
         ValueConv => q{
             my $len = ord($val);
-            $val = substr($val, 1, $len) if $len < length($val);
-            return $val;
+            $val = substr($val, 0, $len+1) if $len < length($val);
+            return Image::ExifTool::Photoshop::ConvertPascalString($self,$val);
         },
-        # there are 6 bytes of unknown information that follow the name
     },
     0x2710 => { Unknown => 1, Name => 'PrintFlagsInfo' },
 );
@@ -308,6 +310,7 @@ my %psdMap = (
     11 => 'BitDepth',
     12 => {
         Name => 'ColorMode',
+        PrintConvColumns => 2,
         PrintConv => {
             0 => 'Bitmap',
             1 => 'Grayscale',
@@ -342,9 +345,9 @@ sub AUTOLOAD
 # Convert pascal string(s) to something we can use
 # Inputs: 1) Pascal string data
 # Returns: Strings, concatenated with ', '
-sub ConvertPascalString($)
+sub ConvertPascalString($$)
 {
-    my $inStr = shift;
+    my ($exifTool, $inStr) = @_;
     my $outStr = '';
     my $len = length($inStr);
     my $i=0;
@@ -355,7 +358,8 @@ sub ConvertPascalString($)
         $outStr .= substr($inStr, $i+1, $n);
         $i += $n + 1;
     }
-    return $outStr;
+    my $charset = $exifTool->Options('CharsetPhotoshop') || 'Latin';
+    return $exifTool->Decode($outStr, $charset);
 }
 
 #------------------------------------------------------------------------------
@@ -565,7 +569,7 @@ be preserved when copying Photoshop information via user-defined tags.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
